/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.fleet.model;

import java.beans.PropertyEditor;
import java.beans.PropertyEditorSupport;
import java.io.Serializable;
import java.lang.reflect.InvocationTargetException;
import org.openide.nodes.Node;

/**
 *
 * @author Wytse Visser
 */
public class MachineProperty<T> implements Serializable {

    private Class<T> valueType;
    private String name;
    private T value;
    private String[] possibleValues;
    
    public MachineProperty(Class<T> valueType, String name, T defaultValue) {
        this.valueType = valueType;
        this.name = name;
        this.value = defaultValue;
    }
    
    public T getValue() throws IllegalAccessException, InvocationTargetException {
        return value;
    }

    public final void setValue(T val) throws IllegalAccessException, IllegalArgumentException, InvocationTargetException {
        this.value = val;
    }

    public void setPossibleValues(String[] possibleValues) {
        this.possibleValues = possibleValues;
    }
    
    public String getName() {
        return this.name;
    }

    public Node.Property<T> getProperty() {
        return new Node.Property<T>(MachineProperty.this.valueType) {

            @Override
            public boolean canRead() {
                return true;
            }

            @Override
            public T getValue() throws IllegalAccessException, InvocationTargetException {
                return value;
            }

            @Override
            public boolean canWrite() {
                return true;
            }

            @Override
            public void setValue(T val) throws IllegalAccessException, IllegalArgumentException, InvocationTargetException {
                MachineProperty.this.value = val;
            }

            @Override
            public String getName() {
                return MachineProperty.this.getName();
            }

            @Override
            public PropertyEditor getPropertyEditor() {
                final String[] tags = possibleValues;
                if (tags == null || tags.length == 0) {
                    return super.getPropertyEditor();
                } else {
                    return new PropertyEditorSupport() {

                        @Override
                        public String[] getTags() {
                            return tags;
                        }
                    };
                }
            }
        };
    }
}