/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.fleet.model;

import java.awt.Image;
import java.util.logging.Logger;
import nl.cloudfarming.client.model.EnterpriseIdProvider;
import nl.cloudfarming.client.model.ItemIdType;
import org.openide.util.ImageUtilities;

/**
 *
 * @author Wytse Visser
 */
public class Vehicle extends Machine {

    private static final Logger LOGGER = Logger.getLogger(Vehicle.class.getName());
    private transient VehicleGeographical vehicleGeographical;
    private int horsepower;
    private int fuelTankCapacity;
    private double displacement;
    public static final String PROP_HORSEPOWER = "horsepower";
    public static final String PROP_DISPLACEMENT = "displacement";
    public static final String PROP_FUELTANKCAPACITY = "fuelTankCapacity";
    public static final ItemIdType ITEM_ID_TYPE = ItemIdType.VCE;
    public static final String EXT = "vce";

    public Vehicle(EnterpriseIdProvider eip, String name) {
        super(eip, ITEM_ID_TYPE, name);
    }

    /**
     * Get the value of fuelTankCapacity
     *
     * @return the value of fuelTankCapacity
     */
    public int getFuelTankCapacity() {
        return fuelTankCapacity;
    }

    /**
     * Set the value of fuelTankCapacity
     *
     * @param fuelTankCapacity new value of fuelTankCapacity
     */
    public void setFuelTankCapacity(int fuelTankCapacity) {
        int oldFuelTankCapacity = this.fuelTankCapacity;
        this.fuelTankCapacity = fuelTankCapacity;
        getPropertyChangeSupport().firePropertyChange(PROP_FUELTANKCAPACITY, oldFuelTankCapacity, fuelTankCapacity);
    }

    /**
     * Get the value of displacement
     *
     * @return the value of displacement
     */
    public double getDisplacement() {
        return displacement;
    }

    /**
     * Set the value of displacement
     *
     * @param displacement new value of displacement
     */
    public void setDisplacement(double displacement) {
        double oldDisplacement = this.displacement;
        this.displacement = displacement;
        getPropertyChangeSupport().firePropertyChange(PROP_DISPLACEMENT, oldDisplacement, displacement);
    }

    /**
     * Get the value of horsepower
     *
     * @return the value of horsepower
     */
    public int getHorsepower() {
        return horsepower;
    }

    /**
     * Set the value of horsepower
     *
     * @param horsepower new value of horsepower
     */
    public void setHorsepower(int horsepower) {
        int oldHorsepower = this.horsepower;
        this.horsepower = horsepower;
        getPropertyChangeSupport().firePropertyChange(PROP_HORSEPOWER, oldHorsepower, horsepower);
    }

    @Override
    public MachineGeometrical getGeometrical() {
        if (vehicleGeographical == null) {
            vehicleGeographical = new VehicleGeographical();
        }
        return vehicleGeographical;
    }

    private class VehicleGeographical extends MachineGeometrical {

        public VehicleGeographical() {
            super(Vehicle.this);
        }

        @Override
        public Image getIcon() {
            return ImageUtilities.loadImage("nl/cloudfarming/client/fleet/vehicle/steering_wheel_332.png");
        }
    }
}
